#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.


from idb.grpc.types import CompanionClient
from idb.grpc.idb_pb2 import MkdirRequest, MkdirResponse


async def client(client: CompanionClient, bundle_id: str, path: str) -> None:
    await client.stub.mkdir(MkdirRequest(bundle_id=bundle_id, path=path))


async def daemon(client: CompanionClient, request: MkdirRequest) -> MkdirResponse:
    return await client.stub.mkdir(request)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.


import asyncio
import os
from logging import Logger
from typing import AsyncIterator, Dict, List, Optional

from idb.grpc.types import CompanionClient
from idb.grpc.stream import Stream
from idb.common.tar import drain_untar
from idb.grpc.idb_pb2 import InstrumentsRunRequest, InstrumentsRunResponse
from idb.utils.typing import none_throws


Start = InstrumentsRunRequest.Start
Stop = InstrumentsRunRequest.Stop


async def _generate_bytes(
    stream: Stream[InstrumentsRunRequest, InstrumentsRunResponse], logger: Logger
) -> AsyncIterator[bytes]:
    async for response in stream:
        log_output = response.log_output
        if len(log_output):
            logger.info(log_output.decode().strip())
            continue
        if response.state == InstrumentsRunResponse.POST_PROCESSING:
            logger.info("Instruments is post processing")
            continue
        data = response.payload.data
        if len(data):
            yield data


async def _drain_until_running(
    stream: Stream[InstrumentsRunRequest, InstrumentsRunResponse], logger: Logger
) -> None:
    while True:
        response = none_throws(await stream.recv_message())
        log_output = response.log_output
        if len(log_output):
            logger.info(log_output.decode().strip())
            continue
        state = response.state
        if state == InstrumentsRunResponse.RUNNING_INSTRUMENTS:
            logger.info("State changed to RUNNING_INSTRUMENTS")
            return


async def _drain_until_stop(
    stream: Stream[InstrumentsRunRequest, InstrumentsRunResponse],
    stop: asyncio.Future,
    logger: Logger,
) -> None:
    while True:
        read = asyncio.ensure_future(stream.recv_message())
        done, pending = await asyncio.wait(
            [stop, read], return_when=asyncio.FIRST_COMPLETED
        )
        if stop in done:
            return
        response = none_throws(read.result())
        output = response.log_output
        if len(output):
            logger.info(output.decode())


async def run_instruments(
    client: CompanionClient,
    stop: asyncio.Event,
    template: str,
    app_bundle_id: str,
    trace_path: str,
    post_process_arguments: Optional[List[str]] = None,
    env: Optional[Dict[str, str]] = None,
    app_args: Optional[List[str]] = None,
    started: Optional[asyncio.Event] = None,
) -> str:
    trace_path = os.path.realpath(trace_path)
    client.logger.info(f"Starting instruments connection, writing to {trace_path}")
    async with client.stub.instruments_run.open() as stream:
        client.logger.info("Sending instruments request")
        await stream.send_message(
            InstrumentsRunRequest(
                start=Start(
                    file_path=None,
                    template_name=template,
                    app_bundle_id=app_bundle_id,
                    environment=env,
                    arguments=app_args,
                )
            )
        )
        client.logger.info("Starting instruments")
        await _drain_until_running(stream=stream, logger=client.logger)
        if started:
            started.set()
        client.logger.info("Instruments has started, waiting for stop")
        await _drain_until_stop(
            stream=stream, stop=asyncio.ensure_future(stop.wait()), logger=client.logger
        )
        client.logger.info("Stopping instruments")
        await stream.send_message(
            InstrumentsRunRequest(
                stop=Stop(post_process_arguments=post_process_arguments)
            ),
            end=True,
        )
        client.logger.info(f"Writing instruments from tar to {trace_path}")
        await drain_untar(
            _generate_bytes(stream=stream, logger=client.logger), output_path=trace_path
        )
        client.logger.info(f"Instruments trace written to {trace_path}")
        return trace_path


CLIENT_PROPERTIES = [run_instruments]  # pyre-ignore



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import json
from argparse import ArgumentParser, Namespace


from idb.cli.commands.base import TargetCommand
from idb.client.client import IdbClient


class ListTestBundleCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "List the tests inside an installed test bundle"

    @property
    def name(self) -> str:
        return "list-bundle"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "test_bundle_id", help="Bundle id of the test bundle to list", type=str
        )
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        tests = await client.list_test_bundle(test_bundle_id=args.test_bundle_id)
        if args.json:
            print(json.dumps(tests))
        else:
            print("\n".join(tests))



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from typing import List

from idb.grpc.types import CompanionClient
from idb.common.types import AppProcessState, InstalledAppInfo
from idb.grpc.idb_pb2 import ListAppsRequest


async def client(client: CompanionClient) -> List[InstalledAppInfo]:
    response = await client.stub.list_apps(ListAppsRequest())
    return [
        InstalledAppInfo(
            bundle_id=app.bundle_id,
            name=app.name,
            architectures=app.architectures,
            install_type=app.install_type,
            process_state=AppProcessState(app.process_state),
            debuggable=app.debuggable,
        )
        for app in response.apps
    ]



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import asyncio
import json
import logging
import os
from asyncio import StreamReader
from asyncio.futures import Future
from typing import Dict, Optional

from idb.common.constants import IDB_LOGS_PATH
from idb.manager.companion import CompanionManager
from idb.common.types import Server, TargetDescription
from idb.utils.typing import none_throws


class CompanionTailerException(Exception):
    pass


class CompanionTailer(Server):
    def __init__(self, notifier_path: str, companion_manager: CompanionManager) -> None:
        self.notifier_path = notifier_path
        self._reading_forever_fut: Future[None]
        self.companion_manager = companion_manager
        self.process: Optional[asyncio.subprocess.Process] = None

    async def _read_stream(self, stream: StreamReader) -> None:
        while True:
            line = await stream.readline()
            if line:
                update = json.loads(line.decode())
                if "initial_state_ended" in update:
                    end_of_initial_state = update["initial_state_ended"]
                    if end_of_initial_state:
                        logging.debug(f"Initial state received from notifier")
                        break
                    else:
                        raise CompanionTailerException(
                            "Unexpected output from companion"
                        )
                self.companion_manager.update_target(
                    TargetDescription(
                        udid=update["udid"],
                        name=update["name"],
                        state=update["state"],
                        target_type=update["type"],
                        os_version=update["os_version"],
                        architecture=update["architecture"],
                        companion_info=None,
                        screen_dimensions=None,
                    )
                )
            else:
                break

    def _log_file_path(self) -> str:
        os.makedirs(name=IDB_LOGS_PATH, exist_ok=True)
        return IDB_LOGS_PATH + "/notifier"

    async def notifierProcess(self) -> asyncio.subprocess.Process:
        cmd = [self.notifier_path, "--notify", "1"]
        with open(self._log_file_path(), "a") as log_file:
            process = await asyncio.create_subprocess_exec(
                *cmd, stdout=asyncio.subprocess.PIPE, stderr=log_file
            )
            await self._read_stream(none_throws(process.stdout))
            return process

    async def start(self) -> None:
        logging.debug(f"Started tailing notifier")
        if self.process:
            logging.warning(f"Trying to start companion tailer when already running")
            return
        self.process = await self.notifierProcess()
        if self.process:
            self._reading_forever_fut = asyncio.ensure_future(
                self._read_stream(stream=none_throws(self.process.stdout))
            )

    def close(self) -> None:
        logging.debug("Stopping companion tailer")
        self._reading_forever_fut.cancel()
        if self.process:
            self.process.terminate()

    async def wait_closed(self) -> None:
        pass

    @property
    def ports(self) -> Dict[str, int]:
        return {}



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from typing import List

from idb.grpc.types import CompanionClient
from idb.grpc.idb_pb2 import RmRequest, RmResponse


async def client(client: CompanionClient, bundle_id: str, paths: List[str]) -> None:
    await client.stub.rm(RmRequest(bundle_id=bundle_id, paths=paths))


async def daemon(client: CompanionClient, request: RmRequest) -> RmResponse:
    return await client.stub.rm(request)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from unittest import mock

from idb.daemon.server import CompositeServer
from idb.utils.testing import TestCase, ignoreTaskLeaks


@ignoreTaskLeaks
class ServerTest(TestCase):
    def test_close(self) -> None:
        first = mock.MagicMock()
        second = mock.MagicMock()
        server = CompositeServer(servers=[first, second], logger=mock.MagicMock())
        server.close()
        first.close.assert_called_once()
        second.close.assert_called_once()



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import json
from argparse import ArgumentParser, Namespace, SUPPRESS
from typing import Union

from idb.cli.commands.base import ConnectingCommand
from idb.client.client import IdbClient
from idb.common.types import Address, IdbException
import idb.common.plugin as plugin


def get_destination(args: Namespace) -> Union[Address, str]:
    target_udid = args.companion if "-" in args.companion else None
    companion_host = args.companion if not target_udid else None
    if target_udid:
        return target_udid
    elif args.port and args.grpc_port and companion_host:
        return Address(host=companion_host, port=args.port, grpc_port=args.grpc_port)
    elif args.port and companion_host:
        return Address(host=companion_host, grpc_port=args.port)
    else:
        raise ConnectCommandException(
            "provide either a UDID or the host and port of the companion"
        )


class ConnectCommandException(Exception):
    pass


class ConnectCommand(ConnectingCommand):
    @property
    def description(self) -> str:
        return "Connect to a companion"

    @property
    def name(self) -> str:
        return "connect"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "companion",
            help="Host the companion is running on. or the UDID of the target",
            type=str,
        )
        parser.add_argument(
            "port",
            help="Port the companion is running on",
            type=int,
            nargs="?",
            default=None,
        )
        parser.add_argument(
            "grpc_port", help=SUPPRESS, type=int, nargs="?", default=None
        )
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        try:
            destination = get_destination(args=args)
            connect_response = await client.connect(
                destination=destination,
                metadata={
                    key: value
                    for (key, value) in plugin.resolve_metadata(self.logger).items()
                    if isinstance(value, str)
                },
            )
            if connect_response:
                if args.json:
                    print(
                        json.dumps(
                            {
                                "udid": connect_response.udid,
                                "is_local": connect_response.is_local,
                            }
                        )
                    )
                else:
                    print(
                        f"udid: {connect_response.udid} is_local: {connect_response.is_local}"
                    )

        except IdbException:
            raise ConnectCommandException(
                f"""Could not connect to {args.companion:}:{args.port}.
            Make sure both host and port are correct and reachable"""
            )



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import asyncio
import json
import logging
import os
import platform
import sys
from asyncio import StreamReader
from typing import List

import idb.common.networking as networking
from idb.client.daemon_pid_saver import kill_saved_pids, save_daemon_pid
from idb.common.constants import (
    DEFAULT_DAEMON_HOST,
    DEFAULT_DAEMON_GRPC_PORT,
    IDB_LOGS_PATH,
)
from idb.utils.typing import none_throws


class DaemonSpawnerException(Exception):
    pass


class DaemonSpawner:
    def __init__(self, port: int, host: str) -> None:
        self.port: int = port
        self.host: str = host

    async def start_daemon_if_needed(self, force_kill: bool = False) -> None:
        if self.port != DEFAULT_DAEMON_GRPC_PORT or self.host != DEFAULT_DAEMON_HOST:
            # don't spawn a daemon if there's any overrides
            return
        if not networking.is_port_open(self.host, self.port):
            await self._spawn_daemon()
        elif force_kill:
            await kill_saved_pids()
            await self._spawn_daemon()

    async def _read_daemon_output(self, stream: StreamReader) -> None:
        line = await stream.readline()
        try:
            json.loads(line.decode())
        except json.decoder.JSONDecodeError as e:
            raise DaemonSpawnerException(f"Failed to spawn daemon: {line}") from e

    def _log_file_path(self) -> str:
        os.makedirs(name=IDB_LOGS_PATH, exist_ok=True)
        return IDB_LOGS_PATH + "/daemon"

    async def _spawn_daemon(self) -> None:
        cmd: List[str] = [sys.argv[0], "daemon"]
        if platform.system() == "Darwin":
            logging.debug("Mac Detected. passing notifier path to daemon")
            cmd.extend(["--notifier-path", "idb_companion"])
        with open(self._log_file_path(), "w") as log_file:
            process = await asyncio.create_subprocess_exec(
                *cmd, stdout=asyncio.subprocess.PIPE, stderr=log_file
            )
            logging.debug(f"daemon process id {process.pid}")
            save_daemon_pid(pid=process.pid)
            await self._read_daemon_output(none_throws(process.stdout))



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from typing import List

from idb.grpc.types import CompanionClient
from idb.grpc.stream import Stream, drain_to_stream
from idb.common.stream import stream_map
from idb.common.tar import generate_tar
from idb.grpc.idb_pb2 import Payload, PushRequest, PushResponse


Inner = PushRequest.Inner


async def client(
    client: CompanionClient, src_paths: List[str], bundle_id: str, dest_path: str
) -> None:
    async with client.stub.push.open() as stream:
        await stream.send_message(
            PushRequest(inner=Inner(bundle_id=bundle_id, dst_path=dest_path))
        )
        for src_path in src_paths:
            await stream.send_message(PushRequest(payload=Payload(file_path=src_path)))
        await stream.end()
        await stream.recv_message()


async def daemon(
    client: CompanionClient, stream: Stream[PushResponse, PushRequest]
) -> None:
    async with client.stub.push.open() as companion:
        await companion.send_message(await stream.recv_message())
        if client.is_local:
            generator = stream
        else:
            paths = [request.payload.file_path async for request in stream]
            generator = stream_map(
                generate_tar(paths=paths),
                lambda chunk: PushRequest(payload=Payload(data=chunk)),
            )
        response = await drain_to_stream(
            stream=companion, generator=generator, logger=client.logger
        )
        await stream.send_message(response)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import asyncio
import importlib
from argparse import ArgumentParser, Namespace
from logging import Logger
from typing import List, TYPE_CHECKING
from types import ModuleType


from idb.common.types import LoggingMetadata, IdbClientBase, Server


if TYPE_CHECKING:
    from idb.manager.companion import CompanionManager
    from idb.common.boot_manager import BootManager


def package_exists(package_name: str) -> bool:
    try:
        return importlib.util.find_spec(package_name) is not None
    except Exception:
        return False


PLUGIN_PACKAGE_NAMES = ["idb.fb.plugin"]
PLUGINS: List[ModuleType] = [
    importlib.import_module(package.name)
    for package in [
        importlib.util.find_spec(package_name)
        for package_name in PLUGIN_PACKAGE_NAMES
        if package_exists(package_name)
    ]
    if package is not None
]


def on_launch(logger: Logger) -> None:
    for plugin in PLUGINS:
        if not hasattr(plugin, "on_launch"):
            continue
        plugin.on_launch(logger)  # pyre-ignore


async def on_close(logger: Logger) -> None:
    await asyncio.gather(
        *[
            plugin.on_close(logger)  # pyre-ignore
            for plugin in PLUGINS
            if hasattr(plugin, "on_close")
        ]
    )


async def before_invocation(name: str, metadata: LoggingMetadata) -> None:
    await asyncio.gather(
        *[
            plugin.before_invocation(name=name, metadata=metadata)  # pyre-ignore
            for plugin in PLUGINS
            if hasattr(plugin, "before_invocation")
        ]
    )


async def after_invocation(name: str, duration: int, metadata: LoggingMetadata) -> None:
    await asyncio.gather(
        *[
            plugin.after_invocation(  # pyre-ignore
                name=name, duration=duration, metadata=metadata
            )
            for plugin in PLUGINS
            if hasattr(plugin, "after_invocation")
        ]
    )


async def failed_invocation(
    name: str, duration: int, exception: Exception, metadata: LoggingMetadata
) -> None:
    await asyncio.gather(
        *[
            plugin.failed_invocation(  # pyre-ignore
                name=name, duration=duration, exception=exception, metadata=metadata
            )
            for plugin in PLUGINS
            if hasattr(plugin, "failed_invocation")
        ]
    )


def on_connecting_parser(parser: ArgumentParser, logger: Logger) -> None:
    for plugin in PLUGINS:
        plugin_parser = getattr(plugin, "on_connecting_parser", None)
        if parser is None:
            continue
        plugin_parser(parser=parser, logger=logger)


def resolve_client(
    args: Namespace, logger: Logger, client: IdbClientBase, name: str
) -> IdbClientBase:
    for plugin in PLUGINS:
        plugin_resolver = getattr(plugin, "resolve_client", None)
        if not plugin_resolver:
            continue
        client = plugin_resolver(args=args, logger=logger, client=client, name=name)
    return client


def resolve_metadata(logger: Logger) -> LoggingMetadata:
    metadata = {}
    for plugin in PLUGINS:
        plugin_resolver = getattr(plugin, "resolve_metadata", None)
        if not plugin_resolver:
            continue
        resolved = plugin_resolver(logger=logger)
        metadata.update(resolved)
    return metadata


async def resolve_servers(
    args: Namespace,
    companion_manager: "CompanionManager",
    boot_manager: "BootManager",
    logger: Logger,
    servers: List[Server],
) -> List[Server]:
    for plugin in PLUGINS:
        plugin_resolver = getattr(plugin, "resolve_servers", None)
        if not plugin_resolver:
            continue
        resolved_servers = await plugin_resolver(
            args=args,
            companion_manager=companion_manager,
            boot_manager=boot_manager,
            logger=logger,
            servers=servers,
        )
        servers = servers + resolved_servers
    return servers



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import json
from argparse import ArgumentParser, Namespace

from idb.cli.commands.base import TargetCommand
from idb.client.client import IdbClient
from idb.common.types import CrashLogQuery


class CrashDeleteException(Exception):
    pass


def _add_query_arguments(parser: ArgumentParser) -> None:
    parser.add_argument(
        "--before", help="Match based older than the provided unix timestamp", type=int
    )
    parser.add_argument(
        "--since",
        help="Match based on being newer than the provided unix timestamp",
        type=int,
    )
    parser.add_argument(
        "--bundle-id",
        help="Filter based on the bundle id of the crashed process",
        type=str,
    )


def _build_query(arguments: Namespace) -> CrashLogQuery:
    if (
        hasattr(arguments, "all")
        and not arguments.all
        and hasattr(arguments, "name")
        and not arguments.name
        and arguments.before is None
        and arguments.since is None
        and arguments.bundle_id is None
    ):
        raise CrashDeleteException("Must pass --all if not other arguments specified")

    return CrashLogQuery(
        before=arguments.before,
        since=arguments.since,
        bundle_id=arguments.bundle_id,
        name=getattr(arguments, "name", None),
    )


class CrashListCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "List the available crashes"

    @property
    def name(self) -> str:
        return "list"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        _add_query_arguments(parser=parser)
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        crashes = await client.crash_list(query=_build_query(args))
        for crash in crashes:
            print(json.dumps(crash._asdict()))


class CrashShowCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Fetch a crash log"

    @property
    def name(self) -> str:
        return "show"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument("name", help="The unique name of the crash")
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        crash = await client.crash_show(name=args.name)
        print(crash.contents)


class CrashDeleteCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Delete a crash log"

    @property
    def name(self) -> str:
        return "delete"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "name", nargs="?", default=None, help="The unique name of the crash"
        )
        _add_query_arguments(parser=parser)
        parser.add_argument("--all", help="Delete all crash logs", action="store_true")
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        crashes = await client.crash_delete(query=_build_query(args))
        for crash in crashes:
            print(json.dumps(crash._asdict()))



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from idb.grpc.types import CompanionClient
from idb.grpc.idb_pb2 import FocusRequest


async def client(client: CompanionClient) -> None:
    await client.stub.focus(FocusRequest())



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import asyncio
from typing import Dict, Optional

from logging import Logger

from idb.grpc.handler import GRPCHandler
from idb.common.types import Server
from idb.utils.typing import none_throws
from idb.common.socket import ports_from_sockets
from grpclib.server import Server as GRPC_Server


class GRPCServer(GRPC_Server, Server):
    def __init__(self, handler: GRPCHandler, logger: Logger) -> None:
        GRPC_Server.__init__(self, [handler], loop=asyncio.get_event_loop())
        self.logger = logger

    @property
    def ports(self) -> Dict[str, Optional[int]]:
        (ipv4, ipv6) = ports_from_sockets(none_throws(self._server).sockets)
        return {"ipv4_grpc_port": ipv4, "ipv6_grpc_port": ipv6}



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from typing import Dict, Optional

from idb.grpc.types import CompanionClient
from idb.grpc.ipc_loader import DaemonContext
from idb.common.types import ConnectionDestination, ConnectResponse
from idb.grpc.idb_pb2 import (
    ConnectRequest as GrpcConnectRequest,
    ConnectResponse as GrpcConnectResponse,
)
from idb.ipc.mapping.companion import companion_to_grpc, companion_to_py
from idb.ipc.mapping.destination import destination_to_grpc, destination_to_py
from idb.utils.typing import none_throws


async def client(
    client: CompanionClient,
    destination: ConnectionDestination,
    metadata: Optional[Dict[str, str]] = None,
) -> ConnectResponse:
    client.logger.debug(f"Connecting to {destination} with meta {metadata}")
    response = await client.stub.connect(
        GrpcConnectRequest(
            destination=destination_to_grpc(destination), metadata=metadata
        )
    )
    return companion_to_py(response.companion)


async def daemon(
    context: DaemonContext, request: GrpcConnectRequest
) -> GrpcConnectResponse:
    destination = destination_to_py(none_throws(request.destination))
    return await connect_companion(context, destination, request.metadata)


async def connect_companion(
    context: DaemonContext,
    destination: ConnectionDestination,
    metadata: Optional[Dict[str, str]],
) -> GrpcConnectResponse:
    async with context.companion_manager.create_companion_for_target_with_destination(
        destination=destination, metadata=metadata
    ) as companion:
        return GrpcConnectResponse(companion=companion_to_grpc(companion))



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.


from idb.grpc.types import CompanionClient
from idb.grpc.idb_pb2 import OpenUrlRequest


async def client(client: CompanionClient, url: str) -> None:
    await client.stub.open_url(OpenUrlRequest(url=url))



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import asyncio
import logging
import warnings
from typing import Dict, Optional

import idb.grpc.ipc_loader as ipc_loader
from grpclib.client import Channel
from idb.grpc.types import CompanionClient
from idb.grpc.idb_grpc import CompanionServiceStub
from idb.common.types import IdbClientBase
from idb.client.daemon_spawner import DaemonSpawner

# this is to silence the channel not closed warning
# https://github.com/vmagamedov/grpclib/issues/58
warnings.filterwarnings(action="ignore", category=ResourceWarning)


class IdbGRPCClient(IdbClientBase):
    def __init__(
        self,
        port: int,
        host: str,
        target_udid: Optional[str],
        logger: Optional[logging.Logger] = None,
        force_kill_daemon: bool = False,
    ) -> None:
        self.port: int = port
        self.host: str = host
        self.logger: logging.Logger = (
            logger if logger else logging.getLogger("idb_grpc_client")
        )
        self.force_kill_daemon = force_kill_daemon
        self.target_udid = target_udid
        self.daemon_spawner = DaemonSpawner(host=self.host, port=self.port)
        self.channel: Optional[Channel] = None
        self.stub: Optional[CompanionServiceStub] = None
        for (call_name, f) in ipc_loader.client_calls(
            daemon_provider=self.provide_client
        ):
            setattr(self, call_name, f)

    async def provide_client(self) -> CompanionClient:
        await self.daemon_spawner.start_daemon_if_needed(
            force_kill=self.force_kill_daemon
        )
        if not self.channel or not self.stub:
            self.channel = Channel(self.host, self.port, loop=asyncio.get_event_loop())
            self.stub = CompanionServiceStub(channel=self.channel)
        return CompanionClient(
            stub=self.stub, is_local=True, udid=self.target_udid, logger=self.logger
        )

    @property
    def metadata(self) -> Dict[str, str]:
        if self.target_udid:
            return {"udid": self.target_udid}
        else:
            return {}



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.


from idb.grpc.types import CompanionClient
from idb.grpc.idb_pb2 import Location, SetLocationRequest


async def client(client: CompanionClient, latitude: float, longitude: float) -> None:
    await client.stub.set_location(
        SetLocationRequest(location=Location(latitude=latitude, longitude=longitude))
    )



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from argparse import ArgumentParser, Namespace


from idb.cli.commands.base import TargetCommand
from idb.client.client import IdbClient
from idb.common.misc import get_env_with_idb_prefix
from idb.common.signal import signal_handler_event


class InstrumentsCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Run instruments on the device"

    @property
    def name(self) -> str:
        return "instruments"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "template", help="Template to run (see Apple for possible values)", type=str
        )
        parser.add_argument(
            "--app-bundle-id", help="App to run instruments on (Unsupported)", type=str
        )
        parser.add_argument(
            "--trace-path", help="Path where the trace file will be saved", type=str
        )
        parser.add_argument(
            "--post-args",
            nargs="*",
            help="Post processing arguments to process the Instruments trace",
        )
        parser.add_argument(
            "--app-args",
            nargs="*",
            help="Arguments to be passed to the app being profiled",
        )
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        trace_path = args.trace_path
        if trace_path is None:
            trace_path = f"{args.template}.trace"
        app_args = None if not args.app_args else args.app_args
        post_process_arguments = None if not args.post_args else args.post_args
        result = await client.run_instruments(
            stop=signal_handler_event("instruments"),
            template=args.template,
            app_bundle_id=args.app_bundle_id,
            post_process_arguments=post_process_arguments,
            env=get_env_with_idb_prefix(),
            app_args=app_args,
            trace_path=trace_path,
        )
        print(result)
